/**
 * ProcessLDE - Instruction Length Disassembler Engine
 *
 * This program measures the length of x86/x64 assembly instructions
 * using a debugging technique to analyze instruction boundaries.
 */

#pragma comment(linker, "/ENTRY:WMain")
#pragma comment(linker, "/NODEFAULTLIB")
#pragma comment(linker, "/SUBSYSTEM:CONSOLE")
#pragma comment(linker, "/merge:.rdata=.text")
#pragma comment(linker, "/merge:.data=.text")
#pragma comment(linker, "/SECTION:.text,EWR")

#include <windows.h>

// External Windows API function
#undef RtlMoveMemory
extern VOID __stdcall RtlMoveMemory(IN VOID UNALIGNED *Destination, IN CONST VOID UNALIGNED *Source, IN SIZE_T Length);

// Architecture detection
#ifdef _WIN64
#define ARCHITECTURE_X64
#else
#define ARCHITECTURE_X86
#endif

// Architecture-specific types
#ifdef ARCHITECTURE_X64
typedef DWORD64 MEMORY_ADDRESS;
#define REG_IP Rip
#else
typedef DWORD MEMORY_ADDRESS;
#define REG_IP Eip
#endif

/**
 * Gets the path of the current executable
 *
 * @param buffer Buffer to store the path
 * @param size Size of the buffer
 * @return TRUE if successful, FALSE otherwise
 */
BOOL GetCurrentModulePath(char* buffer, DWORD size)
{
	DWORD result = GetModuleFileNameA(NULL, buffer, size);
	return (result > 0 && result < size);
}

/**
 * Analyzes the length of an instruction at the specified address
 *
 * @param Address Pointer to the instruction to analyze
 * @return Length of the instruction in bytes
 */
BYTE LDE(PVOID Address)
{
	// Debug process variables
	CONTEXT context;
	DEBUG_EVENT DBEvent;
	STARTUPINFOA startinfo;
	PROCESS_INFORMATION pi;
	HANDLE hThread, hProcess;
	BYTE CodeSize = 0;
	MEMORY_ADDRESS Region;
	BOOL Started = FALSE;
	SIZE_T dwWritten;
	char modulePath[MAX_PATH];

	// Initialize structures using direct assignment
	context.ContextFlags = 0;

	// Initialize STARTUPINFO structure
	startinfo.cb = sizeof(STARTUPINFOA);
	startinfo.lpReserved = NULL;
	startinfo.lpDesktop = NULL;
	startinfo.lpTitle = NULL;
	startinfo.dwX = 0;
	startinfo.dwY = 0;
	startinfo.dwXSize = 0;
	startinfo.dwYSize = 0;
	startinfo.dwXCountChars = 0;
	startinfo.dwYCountChars = 0;
	startinfo.dwFillAttribute = 0;
	startinfo.dwFlags = 0;
	startinfo.wShowWindow = 0;
	startinfo.cbReserved2 = 0;
	startinfo.lpReserved2 = NULL;
	startinfo.hStdInput = NULL;
	startinfo.hStdOutput = NULL;
	startinfo.hStdError = NULL;

	// Initialize PROCESS_INFORMATION
	pi.hProcess = NULL;
	pi.hThread = NULL;
	pi.dwProcessId = 0;
	pi.dwThreadId = 0;

	// Get the path of the current executable
	if (!GetCurrentModulePath(modulePath, MAX_PATH)) {
		// Fallback to hardcoded name if path retrieval fails
		lstrcpyA(modulePath, "Empty.exe");
	}

	// Create debugged process using current module's path
	CreateProcessA(modulePath, NULL, NULL, NULL, FALSE,
				 DEBUG_PROCESS | DEBUG_ONLY_THIS_PROCESS,
				 NULL, NULL, &startinfo, &pi);

	// Debug event loop
	while (TRUE)
	{
		// Wait for a debug event
		WaitForDebugEvent(&DBEvent, INFINITE);

		// Handle different debug events
		if (DBEvent.dwDebugEventCode == CREATE_PROCESS_DEBUG_EVENT)
		{
			// Store process and thread handles
			hThread = DBEvent.u.CreateProcessInfo.hThread;
			hProcess = DBEvent.u.CreateProcessInfo.hProcess;
		}
		else if (DBEvent.dwDebugEventCode == EXCEPTION_DEBUG_EVENT)
		{
			// Get thread context for analysis
			context.ContextFlags = CONTEXT_CONTROL;
			GetThreadContext(hThread, &context);

			// Handle access violation exception (DEP)
			if ((DBEvent.u.Exception.ExceptionRecord.ExceptionCode == EXCEPTION_ACCESS_VIOLATION) &&
				(DBEvent.u.Exception.ExceptionRecord.ExceptionInformation[0] == 8))
			{
				// Each access violation means we've hit the end of an instruction
				Region -= 1;
				CodeSize += 1;

				// Write the code to analyze to the target process memory
				WriteProcessMemory(hProcess, (LPVOID)Region, Address, CodeSize, &dwWritten);

				// Set trap flag to enable single-step execution
				context.EFlags |= 0x00000100; // TF (Trap Flag)

				// Set instruction pointer to our memory region
				context.REG_IP = Region;

				SetThreadContext(hThread, &context);
			}
			// Handle breakpoint exception for initialization
			else if ((DBEvent.u.Exception.ExceptionRecord.ExceptionCode == EXCEPTION_BREAKPOINT) && (!Started))
			{
				// Allocate memory in the target process:

				// 1. Start with zero address and find available region
				Region = 0;

				// 2. Reserve 2 pages (8KB) with no access permissions
				Region = (MEMORY_ADDRESS)VirtualAllocEx(hProcess, (LPVOID)Region,
											  0x2000, MEM_RESERVE, PAGE_NOACCESS);

				// 3. Commit first page (4KB) with execute/read/write permissions
				Region = (MEMORY_ADDRESS)VirtualAllocEx(hProcess, (LPVOID)Region,
											  0x1000, MEM_COMMIT, PAGE_EXECUTE_READWRITE);

				// 4. Move to second page to trigger access violation
				Region += 0x1000;

				// 5. Set trap flag for single-step execution
				context.EFlags |= 0x00000100; // TF (Trap Flag)

				// 6. Set instruction pointer to our memory region
				context.REG_IP = Region;

				SetThreadContext(hThread, &context);
				Started = TRUE;
			}
			// Handle other exceptions by terminating process
			else
			{
				TerminateProcess(hProcess, 0);
			}
		}
		else if (DBEvent.dwDebugEventCode == EXIT_PROCESS_DEBUG_EVENT)
		{
			break;
		}

		// Continue debugging
		ContinueDebugEvent(DBEvent.dwProcessId, DBEvent.dwThreadId, DBG_CONTINUE);
	}

	return CodeSize;
}

/**
 * Helper function to output a byte array in hexadecimal format
 *
 * @param buffer Output buffer to write the hex string to
 * @param data Byte array to convert
 * @param length Number of bytes to convert
 */
void BytesToHexString(char* buffer, BYTE* data, DWORD length)
{
	char* ptr = buffer;
	*ptr = 0; // Ensure null termination for empty arrays

	for (DWORD i = 0; i < length; i++) {
		ptr += wsprintfA(ptr, "%02X ", data[i]);
	}
}

/**
 * Helper function to analyze and display instruction length
 *
 * @param stdOutHandle Handle to the console output
 * @param name Test name/description
 * @param bytes Byte array to analyze
 * @param size Size of the byte array
 */
void AnalyzeAndDisplayInstruction(HANDLE stdOutHandle, const char* name, BYTE* bytes, DWORD size)
{
	char buffer[512];
	char hexBuffer[256];
	DWORD bytesWritten;
	BYTE length;

	// Display test case information
	wsprintfA(buffer, "Test: %s\r\n", name);
	WriteConsoleA(stdOutHandle, buffer, lstrlenA(buffer), &bytesWritten, 0);

	// Convert bytes to hex string for display
	BytesToHexString(hexBuffer, bytes, size);
	wsprintfA(buffer, "Bytes: %s\r\n", hexBuffer);
	WriteConsoleA(stdOutHandle, buffer, lstrlenA(buffer), &bytesWritten, 0);

	// Process the instruction
	length = LDE(bytes);

	// Display result
	wsprintfA(buffer, "Instruction Length: 0x%02X (%d) bytes\r\n", length, length);
	WriteConsoleA(stdOutHandle, buffer, lstrlenA(buffer), &bytesWritten, 0);

	// Visualize the instruction boundary
	if (length > 0 && length <= size) {
		BytesToHexString(hexBuffer, bytes, length);
		wsprintfA(buffer, "First Instruction: %s\r\n", hexBuffer);
		WriteConsoleA(stdOutHandle, buffer, lstrlenA(buffer), &bytesWritten, 0);

		// Show remaining bytes if any
		if (length < size) {
			BytesToHexString(hexBuffer, bytes + length, size - length);
			wsprintfA(buffer, "Remaining Bytes: %s\r\n", hexBuffer);
			WriteConsoleA(stdOutHandle, buffer, lstrlenA(buffer), &bytesWritten, 0);
		}
	}

	// Separator between test cases
	wsprintfA(buffer, "\r\n---------------------------------------------------\r\n\r\n");
	WriteConsoleA(stdOutHandle, buffer, lstrlenA(buffer), &bytesWritten, 0);
}

/**
 * Loads a function from user32.dll for analysis
 *
 * @param functionName The name of the function to load
 * @param buffer Buffer to store the first bytes of the function
 * @param bufferSize Size of the buffer
 * @return Number of bytes copied, or 0 on failure
 */
DWORD LoadFunctionFromUser32(const char* functionName, BYTE* buffer, DWORD bufferSize)
{
	HMODULE hModule;
	FARPROC procAddress;
	DWORD bytesRead = 0;

	// Initialize the buffer to avoid random data in case of failure
	for (DWORD i = 0; i < bufferSize; i++) {
		buffer[i] = 0;
	}

	// Load user32.dll
	hModule = LoadLibraryA("user32.dll");
	if (hModule == NULL) {
		return 0;
	}

	// Get function address
	procAddress = GetProcAddress(hModule, functionName);
	if (procAddress == NULL) {
		FreeLibrary(hModule);
		return 0;
	}

	// Copy function bytes (without SEH)
	// We'll use a simpler approach without exception handling
	bytesRead = bufferSize;

	// The VirtualQuery call can check if memory is readable before accessing
	MEMORY_BASIC_INFORMATION memInfo;
	if (VirtualQuery(procAddress, &memInfo, sizeof(memInfo)) == sizeof(memInfo)) {
		if (memInfo.State == MEM_COMMIT &&
			(memInfo.Protect & (PAGE_EXECUTE_READ | PAGE_EXECUTE_READWRITE | PAGE_READONLY | PAGE_READWRITE))) {
			// Memory is readable, copy the bytes
			for (DWORD i = 0; i < bytesRead; i++) {
				buffer[i] = ((BYTE*)procAddress)[i];
			}
		} else {
			// Memory is not readable
			bytesRead = 0;
		}
	} else {
		// VirtualQuery failed
		bytesRead = 0;
	}

	// Clean up
	FreeLibrary(hModule);
	return bytesRead;
}

/**
 * Enhanced entry point with advanced testing capabilities
 */
void WMain(void)
{
	// Architecture-specific test cases
	BYTE testx86[] = { 0x8B, 0x84, 0x8B, 0x40, 0x30, 0x20, 0x10 }; // MOV EAX, [EBX+ECX*4+0x10203040]

#ifdef ARCHITECTURE_X64
	// x64-specific test cases
	// REX.W + MOV RAX, [RBX+RCX*4+0x10203040]
	BYTE testx64_1[] = { 0x48, 0x8B, 0x84, 0x8B, 0x40, 0x30, 0x20, 0x10 };

	// REX.W + MOV R8, [R9+R10*8+0x20]
	BYTE testx64_2[] = { 0x4A, 0x8B, 0x44, 0xD1, 0x20 };

	// MOVAPS XMM12, [R14+32] - using x64 extended registers
	BYTE testx64_3[] = { 0x44, 0x0F, 0x28, 0x66, 0x20 };
#endif

	// Common test cases for both architectures
	// Test case 1: INT 3 (breakpoint) - 1 byte instruction
	BYTE test1[] = { 0xCC };

	// Test case 2: Multiple instructions sequence
	BYTE test2[] = { 0xCC, 0xBC, 0x00, 0x00, 0x00, 0x00, 0x8E, 0xD0, 0xA3, 0x00, 0x10, 0x23, 0x00 };

	// Test case 3: PUSH/POP sequence
	BYTE test3[] = { 0x50, 0x51, 0x52, 0x58, 0x59, 0x5A };

	// Test case 4: SSE/MMX instructions
	BYTE test4[] = { 0x0F, 0x28, 0x48, 0x10, 0x0F, 0xD4, 0xC1, 0x0F, 0x5A, 0x9C, 0xD6, 0x78, 0x56, 0x34, 0x12 };

	// Test case 5: Real function from user32.dll (MessageBoxA)
	BYTE test5[32];
	DWORD test5Size = LoadFunctionFromUser32("MessageBoxA", test5, sizeof(test5));

	// Console variables
	HANDLE stdOutHandle;
	HANDLE stdInHandle;
	DWORD bytesWritten;
	char buffer[512];

	// Set up console interface
#ifdef ARCHITECTURE_X64
	SetConsoleTitleA("Process LDE - 64-bit Instruction Length Analyzer");
#else
	SetConsoleTitleA("Process LDE - 32-bit Instruction Length Analyzer");
#endif

	stdOutHandle = GetStdHandle(STD_OUTPUT_HANDLE);
	stdInHandle = GetStdHandle(STD_INPUT_HANDLE);

	// Display header with architecture information
#ifdef ARCHITECTURE_X64
	wsprintfA(buffer, "===== ProcessLDE: x64 (64-bit) Instruction Length Analysis =====\r\n\r\n");
#else
	wsprintfA(buffer, "===== ProcessLDE: x86 (32-bit) Instruction Length Analysis =====\r\n\r\n");
#endif
	WriteConsoleA(stdOutHandle, buffer, lstrlenA(buffer), &bytesWritten, 0);

	// Process common test cases
	AnalyzeAndDisplayInstruction(stdOutHandle, "INT 3 (Breakpoint)", test1, sizeof(test1));
	AnalyzeAndDisplayInstruction(stdOutHandle, "x86 MOV EAX, [EBX+ECX*4+0x10203040]", testx86, sizeof(testx86));
	AnalyzeAndDisplayInstruction(stdOutHandle, "Multiple instruction sequence", test2, sizeof(test2));
	AnalyzeAndDisplayInstruction(stdOutHandle, "PUSH/POP sequence", test3, sizeof(test3));
	AnalyzeAndDisplayInstruction(stdOutHandle, "SSE/MMX Instructions", test4, sizeof(test4));

#ifdef ARCHITECTURE_X64
	// x64-specific test cases
	AnalyzeAndDisplayInstruction(stdOutHandle, "x64 REX.W + MOV RAX, [RBX+RCX*4+0x10203040]", testx64_1, sizeof(testx64_1));
	AnalyzeAndDisplayInstruction(stdOutHandle, "x64 REX.W + MOV R8, [R9+R10*8+0x20]", testx64_2, sizeof(testx64_2));
	AnalyzeAndDisplayInstruction(stdOutHandle, "x64 MOVAPS XMM12, [R14+32]", testx64_3, sizeof(testx64_3));
#endif

	// Only display if we successfully loaded the function
	if (test5Size > 0) {
		AnalyzeAndDisplayInstruction(stdOutHandle, "MessageBoxA from user32.dll", test5, test5Size);
	} else {
		wsprintfA(buffer, "Failed to load MessageBoxA from user32.dll\r\n\r\n");
		WriteConsoleA(stdOutHandle, buffer, lstrlenA(buffer), &bytesWritten, 0);

		wsprintfA(buffer, "---------------------------------------------------\r\n\r\n");
		WriteConsoleA(stdOutHandle, buffer, lstrlenA(buffer), &bytesWritten, 0);
	}

	// Display architecture information
#ifdef ARCHITECTURE_X64
	wsprintfA(buffer, "Running in 64-bit mode\r\n");
#else
	wsprintfA(buffer, "Running in 32-bit mode\r\n");
#endif
	WriteConsoleA(stdOutHandle, buffer, lstrlenA(buffer), &bytesWritten, 0);

	// Display completion message
	wsprintfA(buffer, "Analysis complete. Press any key to exit...\r\n");
	WriteConsoleA(stdOutHandle, buffer, lstrlenA(buffer), &bytesWritten, 0);

	// Wait for user input before exiting
	ReadConsoleA(stdInHandle, buffer, 1, &bytesWritten, 0);
}
