# ProcessLDE - Instruction Length Disassembler Engine

## Overview

ProcessLDE is a specialized tool that determines the exact length of x86 and x64 assembly instructions. It works by using Windows debugging mechanisms to analyze how processors interpret assembly code instructions.

This tool is particularly useful for:
- Reverse engineering
- Binary code analysis
- Creating disassemblers
- Developing debuggers or other security tools
- Understanding x86/x64 instruction encoding

## How It Works

ProcessLDE uses a clever approach to determine instruction lengths:

1. It creates a copy of itself as a debugged process
2. It places the instruction bytes to be analyzed at a specific memory location
3. It uses the processor's single-step execution (trap flag)
4. It catches access violations to determine when an instruction has been completely processed
5. It counts how many bytes were executed before reaching the end of the instruction

Unlike static analysis approaches, this technique leverages the CPU itself to make accurate determinations about instruction boundaries, even for complex x86/x64 instructions.

## Features

- **Accurate Measurement**: Uses the actual CPU execution path to determine lengths
- **Dual Architecture Support**: Works in both 32-bit (x86) and 64-bit (x64) modes
- **Supports All x86/x64 Instructions**: Works with simple and complex instructions, including:
  - Basic x86 instructions (PUSH, POP, MOV, etc.)
  - x64-specific instructions with REX prefixes
  - Extended register support (R8-R15, XMM8-XMM15)
  - Instructions with prefixes (REP, LOCK)
  - SIMD instructions (SSE, MMX)
  - Instructions with complex addressing modes
- **Self-Contained**: No external dependencies (uses `/NODEFAULTLIB`)
- **Real-World Testing**: Can analyze real functions from Windows DLLs

## Building and Running

### Prerequisites
- Windows operating system
- Visual Studio 2010 or newer
- Windows SDK

### Building
1. Open the solution (Empty.sln) in Visual Studio
2. Select your desired platform:
   - For 32-bit: Choose "Release|Win32" or "Debug|Win32" configuration
   - For 64-bit: Choose "Release|x64" or "Debug|x64" configuration
3. Build the solution (F7 or Build → Build Solution)
4. The executable will be created in the respective output folder (e.g., Release or x64\Release)

### Running
Simply run the executable directly - no additional setup required. The program will display its results in a console window.

### Understanding the Output

The program runs a series of test cases and displays results like this:

```
Test: INT 3 (Breakpoint)
Bytes: CC
Instruction Length: 0x01 (1) bytes
First Instruction: CC
```

For each test case, ProcessLDE shows:
- The test name
- The raw bytes being analyzed
- The determined length of the first instruction
- The bytes that make up the first instruction
- Any remaining bytes in the sequence

#### Architecture-Specific Output

When running the x64 version, you'll see additional test cases for x64-specific instructions:

```
Test: x64 REX.W + MOV RAX, [RBX+RCX*4+0x10203040]
Bytes: 48 8B 84 8B 40 30 20 10
Instruction Length: 0x08 (8) bytes
First Instruction: 48 8B 84 8B 40 30 20 10
```

Notice how x64 instructions with REX prefixes (like 48h) are properly analyzed and show different lengths than their x86 counterparts.

## How ProcessLDE Is Different

Most instruction length analyzers use:
- Hardcoded tables of opcodes and lengths
- Complex decoding logic that attempts to mimic the CPU

ProcessLDE takes a different approach by letting the CPU itself determine instruction boundaries through direct execution. This is often more accurate, especially for complex instruction encodings.

## Technical Details

### Key Components

1. **Debug Process Creation**:
   - Creates a separate process for instruction analysis
   - Uses `DEBUG_PROCESS | DEBUG_ONLY_THIS_PROCESS` flags

2. **Memory Allocation Strategy**:
   - Allocates two pages: one executable, one protected
   - Places instructions at the boundary to trigger controlled access violations

3. **Single-Step Execution**:
   - Sets the EFLAGS Trap Flag (TF) to enable instruction-by-instruction execution
   - Catches and interprets debug events

4. **Access Violation Handling**:
   - When an access violation occurs at a specific memory location, it indicates the end of an instruction
   - Counts bytes processed to determine instruction length

5. **Architecture Detection**:
   - Automatically detects whether it's running in x86 or x64 mode
   - Adapts behavior and instruction pointer registers (EIP vs RIP) accordingly

### Implementation Notes

- Uses no C/C++ runtime libraries (`/NODEFAULTLIB`)
- Custom entry point (`WMain`)
- Minimal external dependencies
- Optimized for small code size
- Architecture-aware using conditional compilation

### x64 Implementation

In x64 mode, ProcessLDE:
- Uses RIP instead of EIP for instruction pointer
- Uses 64-bit memory addresses
- Handles REX prefixes properly
- Includes test cases for x64-specific instructions
- Correctly analyzes extended register encoding

## Limitations

- Works on Windows only
- x86/x64 instructions only
- Requires that the analyzed instruction can be executed without errors
- Cannot analyze instructions that have side effects (like writing to memory)

## License

This project is provided as-is for educational and research purposes.
